#ifndef MATRIX_HH
#define MATRIX_HH

#include <iostream>
#include <vector>


	//! Collects reusable classes.
namespace biu {


		//! A generic matrix that allows matrix operations.
	template <class T>
	class Matrix
	{
	private:
			//! the matrix dimensions
		unsigned int nn, mm;
			//! the data store
		T **v;
	public:
		Matrix();
			
			//! creates an empty matrix with 0 columns and 0 rows
		Matrix(unsigned int n, unsigned int m);
			//! creates a matrix and fills all elements with a
		Matrix(const T &a, unsigned int n, unsigned int m);
			//! creates a matrix and fills all elements with *a
		Matrix(const T *a, unsigned int n, unsigned int m);
			//! copy constructor including matrix resizing
		Matrix(const Matrix<T> &mat);
			//! assignment including resizing
		Matrix& operator = (const Matrix<T> &mat);
			//! all elements = a
		Matrix& operator = (const T &a);
			//! matrix + matrix
			//! row and column number have to be equal
		Matrix operator + (const Matrix<T> &mat);
			//! matrix + constant
		Matrix operator + (const T &t);
			//! matrix - matrix
			//! row and column number have to be equal
		Matrix operator - (const Matrix<T> &mat);
			//! matrix - constant
		Matrix operator - (const T &t);
			//! matrix * matrix
			//! row number have to equal column number and vice versa
		Matrix operator * (const Matrix<T> &mat) const;
			//! matrix * constant
		Matrix operator * (const T &t) const;
			//! special case of *
			//! column number have to equal vec size
		std::vector<T> operator * (const std::vector<T> &vec) const;
			//! matrix *= matrix
			//! row number have to equal column number and vice versa
		Matrix<T>& operator *= (const Matrix<T> &mat);
			//! matrix *= constant
		Matrix<T>& operator *= (const T& c);
		inline bool operator == (const Matrix<T> &matA) const;
			//! test if all elements == t
		inline bool operator == (const T &t) const;
		inline bool operator != (const Matrix<T> &matA) const;
			//! test if at least one element doesnt equal t
		inline bool operator != (const T &t) const;
			//! access to row i
		inline T* operator[](const unsigned int i);
			//! constant access to row i
		inline const T* operator[](const unsigned int i) const;
			//! access to element (i,j)
		inline T at(const unsigned int i, const unsigned int j);
			//! number of rows
		inline unsigned int numRows() const;
			//! number of columns
		inline unsigned int numColumns() const;
		
		std::vector<T> columnVec(int col) const;
		
			//! resizes the matrix
		void resize(int n, int m);
		~Matrix();

	};


	template <class T> inline
	std::istream&
	operator>>(std::istream& in, const Matrix<T>& m)
	{
		for (unsigned int i = 0; i < m.numRows(); i++)
		{
			for (unsigned int j = 0; j < m.numColumns(); j++)
			{
				in >> m[i][j];
			}
		}
		return in;
	}

	template <class T> inline
	std::ostream&
	operator << (std::ostream& out, const Matrix<T>& m)	{
		for (unsigned int i = 0; i < m.numRows(); i++)
		{
			for (unsigned int j = 0; j < m.numColumns(); j++)
			{
				out << m[i][j] << " ";
			}
			out << std::endl;
		}
		return out;
	}


	template <class T> inline
	Matrix<T>::Matrix()
	{
		nn = 0;
		mm = 0;
		v = 0;
	}

	template <class T> inline
	Matrix<T>::Matrix(unsigned int n, unsigned int m)
	{
		nn = n;
		mm = m;
		v = new T*[nn];
		v[0] = new T[mm*nn];
		for (unsigned int i = 1; i < nn; i++)
			v[i] = v[i-1] + mm;
	}

	template <class T> inline
	Matrix<T>::Matrix(const T &a, unsigned int n, unsigned int m)
	{
		nn = n;
		mm = m;
		v = new T*[nn];
		v[0] = new T[mm*nn];
		for(unsigned int i = 1; i < nn; i++)
			v[i] = v[i-1] + mm;

		for (unsigned int x = 0; x < nn; x++)
			for(unsigned int j = 0; j < mm; j++)
				v[x][j] = a;

	}

	template <class T> inline
	Matrix<T>::Matrix(const T *a, unsigned int n, unsigned int m)
	{

		nn = n;
		mm = m;
		v = new T*[nn];
		v[0] = new T[mm*nn];
		for(int i = 1; i < nn; i++)
			v[i] = v[i-1] + mm;
		for (int x = 0; x < nn; x++)
			for(int j = 0; j < mm; j++)
				v[x][j] = *a++;

	}

	template <class T> inline
	Matrix<T>::Matrix(const Matrix<T> &mat)
	{

		nn = mat.nn;
		mm = mat.mm;
		v = new T*[nn];
		v[0] = new T[mm*nn];
		for (unsigned int i = 1; i < nn; i++)
			v[i] = v[i-1] + mm;

		for (unsigned int x = 0; x < nn; x++)
			for (unsigned int j = 0; j < mm; j++)
				v[x][j] = mat[x][j];

	}

	template <class T> inline
	Matrix<T> & Matrix<T>::operator =(const Matrix<T> &mat)
	{
		if (this != &mat)
		{
			if (nn != mat.nn || mm != mat.mm) {	// neu speicher sichern wenn noetig
				if (v != 0) {
					delete[] (v[0]);
					delete[] (v);
				}
				nn = mat.nn;
				mm = mat.mm;
				v = new T*[nn];
				v[0] = new T[mm*nn];
				for (unsigned int i = 1; i < nn; i++)
					v[i] = v[i-1] + mm;
			}

			for (unsigned int x = 0; x < nn; x++)
			{
				for (unsigned int j = 0; j < mm; j++)
				{
					v[x][j] = mat[x][j];
				}
			}
		}
		return *this;
	}

	template <class T> inline
	Matrix<T> & Matrix<T>::operator =(const T &a)
	{
		for (int i = 0; i < nn; i++)
		{
			for (int j = 0; j < mm; j++)
			{
				v[i][j] = a;
			}
		}
		return *this;
	}

	template <class T> inline
	T* Matrix<T>::operator [](const unsigned int i)
	{
		return v[i];
	}

	template <class T> inline
	const T* Matrix<T>::operator [](const unsigned int i) const
	{
		return v[i];
	}
	template<class T> inline
	T Matrix<T>::at(unsigned int i, unsigned int j)
	{
		return v[i][j];
	}

	template <class T> inline
	unsigned int Matrix<T>::numRows() const
	{
		return nn;
	}

	template <class T> inline
	unsigned int Matrix<T>::numColumns() const
	{
		return mm;
	}

	template <class T> inline
	Matrix<T> Matrix<T>::operator +(const Matrix<T> &mat)
	{
		assert(mat.nn == nn && mat.mm == mm);
		Matrix m(nn, mm);
		for (unsigned int i = 0; i < nn; i++)
			for (unsigned int j = 0; j < mm; j++)
				m[i][j] = v[i][j] + mat[i][j];
		return m;
	}

	template <class T> inline
	Matrix<T> Matrix<T>::operator +(const T &t)
	{
		Matrix m(nn, mm);
		for (unsigned int i = 0; i < nn; i++)
			for (unsigned int j = 0; j < mm; j++)
				m[i][j] = v[i][j] + t;
		return m;
	}

	template <class T> inline
	Matrix<T> Matrix<T>::operator -(const Matrix<T> &mat)
	{
		assert(mat.nn == nn && mat.mm == mm);
		Matrix m(nn, mm);
		for (unsigned int i = 0; i < nn; i++)
			for (unsigned int j = 0; j < mm; j++)
				m[i][j] = v[i][j] - mat[i][j];
			return m;
	}

	template <class T> inline
	Matrix<T> Matrix<T>::operator -(const T &t)
	{
		Matrix m(nn, mm);
		for (unsigned int i = 0; i < nn; i++)
			for (unsigned int j = 0; j < mm; j++)
				m[i][j] = v[i][j] - t;
		return m;
	}

	template <class T> inline
	Matrix<T> Matrix<T>::operator *(const Matrix<T> &mat) const
	{
		assert(mm == mat.nn);	// spaltenzahl muss == mat.zeilenzahl sein
		Matrix m(nn, mat.mm);
		for (unsigned int i = 0; i < nn; i++)
		{
			for (unsigned int j = 0; j < mat.mm; j++)
			{
				m[i][j] = v[i][0] * mat[0][j];
				for (unsigned int k = 1; k < mm; k++)
					m[i][j] += v[i][k] * mat[k][j];
			}
		}
		return m;
	}

	template <class T> inline
	Matrix<T> Matrix<T>::operator *(const T &t) const
	{
		Matrix m(nn, mm);
		for (unsigned int i = 0; i < nn; i++)
			for (unsigned int j = 0; j < mm; j++)
				m[i][j] = v[i][j] * t;
		return m;
	}

	
	template <class T> inline
	std::vector<T> Matrix<T>::operator *(const std::vector<T> &vec) const
	{
		assert(mm == vec.size());	// spaltenzahl muss == vec.size() sein
		
		std::vector<T> res(nn);
		
		for (unsigned int i = 0; i < nn; i++)
		{
			T x=0;
			for (unsigned int j = 0; j < mm; j++) {
				x += v[i][j]*vec[j];
			}
			res[i]=x;
		}
		return res;
	}

	
	template <class T> inline
	Matrix<T>& Matrix<T>::operator*= (const Matrix<T> &mat)
	{
		*this = (*this) * mat;
		return *this;
	}

	template <class T> inline
	Matrix<T>& Matrix<T>::operator*= (const T &t)
	{
		for (unsigned int i = 0; i < nn; i++)
			for (unsigned int j = 0; j < mm; j++)
				v[i][j] *= t;
		return *this;
	}

	template <class T> inline
	bool Matrix<T>::operator ==(const Matrix<T> &matA) const
	{
		unsigned int N = matA.numRows(), M = matA.numColumns(), i, j;
		if (N != this->numRows() || M != this->numColumns())
			return false;

		for(i = 0; i < N; i++)
		{
			for(j = 0; j < M; j++)
			{
				if( matA[i][j] != this->v[i][j] )
					return false;
			}
		}
		return true;
	}

	template <class T> inline
	bool Matrix<T>::operator ==(const T &t) const
	{
		unsigned int N = this->numRows(), M = this->numColumns(), i, j;
		for(i = 0; i < N; i++)
		{
			for(j = 0; j < M; j++)
			{
				if( t != this->v[i][j] )
					return false;
			}
		}
		return true;
	}

	template <class T> inline
	bool Matrix<T>::operator !=(const Matrix<T> &matA) const
	{
		unsigned int N = matA.numRows(), M = matA.numColumns(), i, j;
		if (N != this->numRows() || M != this->numColumns())
			return true;
		for(i = 0; i < N; i++)
		{
			for(j = 0; j < M; j++)
			{
				if( matA[i][j] != this->v[i][j] )
					return true;
			}
		}
		return false;
	}

	template <class T> inline
	bool Matrix<T>::operator !=(const T &t) const
	{
		unsigned int N = this->numRows(), M = this->numColumns(), i, j;
		for(i = 0; i < N; i++)
		{
			for(j = 0; j < M; j++)
			{
				if( t != this->v[i][j] )
					return true;
			}
		}
		return false;
	}

	template <class T> inline
	void Matrix<T>::resize(int n, int m)
	{
		if (n == nn && m == mm)
			return;

		nn = n;
		mm = m;
		if (v != 0)
		{
			delete[] (v[0]);
			delete[] (v);
		}
		v = new T*[nn];
		v[0] = new T[mm*nn];
		for (unsigned int i = 1; i < nn; i++)
			v[i] = v[i-1] + mm;
	}

	template <class T> inline
	Matrix<T>::~Matrix()
	{
		if (v != 0)
		{
			delete[] (v[0]);
			delete[] (v);
		}
	}

	template <class T> inline
	std::vector<T> 
	Matrix<T>::columnVec(int col) const {
		std::vector<T> res(nn);
		for (unsigned int i=0; i<nn; i++) {
			res[i] = v[i][col];
		}
		return res;
	}
	
} // namespace biu

#endif // define MATRIX_HH
